<?php

/** @const SCRY_ID_COOKIE Use cookies to identify. */
define('SCRY_ID_COOKIE', 1);

/** @const SCRY_ID_IP Use ip. */
define('SCRY_ID_IP', 2);

/** @const SCRY_ID_AUTH Use Horde auth. */
define('SCRY_ID_AUTH', 4);

/** @const SCRY_ID_CUSTOM Use a custom string. */
define('SCRY_ID_CUSTOM', 8);

/** @const SCRY_ID_CONFIRM Use an email confirmation. */
define('SCRY_ID_CONFIRM', 16);

/**
 * The Scry:: class provides the Horde Polls system.
 *
 * $Horde: scry/lib/Scry.php,v 1.60 2005/01/03 14:35:27 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.60 $
 * @package Scry
 */
class Scry {

    /**
     * Return the date in the configured format.
     *
     * @param  int    $in  The date in UNIX epoch format.
     *
     * @return string      The date in the $conf.php specified format.
     */
    function getDate($in, $default = null)
    {
        if (empty($in)) {
            if (!is_int($in)) {
                return $default;
            } else {
                $in = $default;
            }
        }
        return strftime($GLOBALS['prefs']->getValue('date_format'), $in);
    }

    /**
     * Work out the current status of the poll and return a text
     * label and a relative class name for the colouring.
     *
     * @param boolean $enabled  Is the poll enabled or not.
     * @param string  $start    The start date in the UNIX epoch format.
     * @param string  $end      The end date in the UNIX epoch format.
     *
     * @return array            Array with two elements, a string with
     *                          the status of the poll and the name of
     *                          css class relative to this status.
     */
    function getStatusLabel($enabled, $start, $end)
    {
        if (!$enabled) {
            $label = _("Disabled");
            $class = 'pollhold';
        } elseif (!is_null($start) && $start > time()) {
            $label= _("Upcoming");
            $class = 'pollnew';
        } elseif (!is_null($end) && $end < time()) {
            $label= _("Finished");
            $class = 'polldone';
        } else {
            $label= _("In progress");
            $class = 'pollactive';
        }
        return array($label, $class);
    }

    /**
     * Returns the $conf allowed set of identifiers.
     *
     * @return array  Array containing as keys allowed identifiers
     *                and their text lables as values.
     */
    function getIdentifiers()
    {
        static $identifiers;
        if (isset($identifiers)) {
            return $identifiers;
        }

        $identifiers = array(
            SCRY_ID_COOKIE  => _("Cookie"),
            SCRY_ID_IP      => _("IP address"),
            SCRY_ID_AUTH    => _("Username"),
            SCRY_ID_CUSTOM  => _("Custom"),
            SCRY_ID_CONFIRM => _("Email confirmation"));

        foreach ($identifiers as $key => $label) {
            if (!in_array($key, $GLOBALS['conf']['polls']['identifiers'])) {
                unset($identifiers[$key]);
            }
        }

        return $identifiers;
    }

    /**
     * Turn an identifier bitmask into an array of identifiers.
     *
     * @param integer $identitfier  The bitmask of identifiers.
     *
     * @return array  An array of each identifier represented by $identitfier.
     */
    function getIdentifierArray($identitfier) 
    {
        $idmasks = array_keys(Scry::getIdentifiers());
        $identifiers = array();
        foreach ($idmasks as $idmask) {
            if ($identitfier & $idmask) {
                $identifiers[$idmask] = $idmask;
            }
        }

        return $identifiers;
    }

    /**
     * Work out the current status of the poll and return HTML
     * formatting the results for display.
     *
     * @param array $votes        The results of the poll in an array.
     *
     * @return string             The formatted display.
     */
    function getResults($votes)
    {
        global $registry;

        /* Sum the votes for a total. */
        $total_votes = array_sum($votes);

        $replies = $total_votes == 1 ? _("1 reply") : sprintf(_("%s replies"), $total_votes);
        $html = '<table width="100%" border="0" cellpadding="1" cellspacing="2" class="text"><tr><th width="40%" align="left">' . _("Choices") . '</th><th width="10%">' . _("Votes") . '</th><th width="10%">%</th><th width="40%" align="left">' . $replies . '</th></tr>';

        /* Get colored and white pixel URLs. */
        $url = Horde::url($registry->get('webroot', 'horde') . '/services/images/pixel.php');
        $left_bar = Util::addParameter($url, 'c', '#bbcbff');
        $right_bar = Util::addParameter($url, 'c', '#ffffff');

        /* Loop and print out each vote. */
        foreach ($votes as $choice => $votes) {
            if ($total_votes > 0) {
                $percent = (int)(($votes / $total_votes) * 100);
            } else {
                $percent = 0;
            }
            $html .= '<tr>';
            $html .= '<td valign="top">' . $choice . '</td>';
            $html .= '<td valign="top" align="center">' . $votes . '</td>';
            $html .= '<td valign="top" align="center">' . $percent . '%</td>';

            $html .= '<td valign="top" nowrap="nowrap">';
            if ($percent != 0 && $percent != 100) {
                $html .= Horde::img($left_bar, '', 'class="leftBar" width="' . $percent . '" height="10"', '');
            }
            if ($percent == 100) {
                $html .= Horde::img($left_bar, '', 'class="rightBar" width="' . $percent . '" height="10"', '');
            } else {
                $html .= Horde::img($right_bar, '', 'class="rightBar" width="' . (100 - $percent) . '" height="10"', '');
            }
            $html .= '</td></tr>';
        }

        $html .= '</table>';
        return $html;
    }

    /**
     * Display the results of the poll including specifying who voted
     * for each response.
     *
     * @param array $votes        The results of the poll in an array.
     * @param array $respondents  Who chose what.
     *
     * @return string  The formatted display.
     */
    function getExtendedResults($votes, $respondents)
    {
        global $registry;

        $total_votes = array_sum($votes);
        $replies = $total_votes == 1 ? _("1 reply") : sprintf(_("%s replies"), $total_votes);
        $html = '<table width="100%" border="0" cellpadding="1" cellspacing="2" class="text"><tr><th width="30%" align="left">' . _("Choices") . '</th><th width="5%">' . _("Votes") . '</th><th width="5%">%</th><th width="30%" align="left">' . $replies . '</th><th width="30%" align="left">' . _("Respondents") . '</th></tr>';

        /* Get colored and white pixel URLs. */
        $url = Horde::url($registry->get('webroot', 'horde') . '/services/images/pixel.php');
        $left_bar = Util::addParameter($url, 'c', '#bbcbff');
        $right_bar = Util::addParameter($url, 'c', '#ffffff');

        /* Loop and print out each vote. */
        foreach ($votes as $choice => $votes) {
            $percent = ($total_votes > 0) ? (int)(($votes / $total_votes) * 100) : 0;
            $html .= '<tr>';
            $html .= '<td valign="top">' . $choice . '</td>';
            $html .= '<td valign="top" align="center">' . $votes . '</td>';
            $html .= '<td valign="top" align="center">' . $percent . '%</td>';

            $html .= '<td valign="top" nowrap="nowrap">';
            if ($percent != 0 && $percent != 100) {
                $html .= Horde::img($left_bar, '', 'class="leftBar" width="' . $percent . '" height="10"', '');
            }
            if ($percent == 100) {
                $html .= Horde::img($left_bar, '', 'class="rightBar" width="' . $percent . '" height="10"', '');
            } else {
                $html .= Horde::img($right_bar, '', 'class="rightBar" width="' . (100 - $percent) . '" height="10"', '');
            }
            $html .= '</td>';

            $html .= '<td valign="top">' . implode('<br />', $respondents[$choice]) . '</td>';
            $html .= '</tr>';
        }

        $html .= '</table>';
        return $html;
    }

    /**
     * Check for permissions on a poll. For now just a place holder
     * since only people with scry:admin permissions are allowed to
     * create/edit/delete.
     *
     * @param  mixed  $in          A poll_id or an array of poll_ids
     *                             to check permission on.
     * @param  string $filter      What type of check to do.
     * @param  string $permission  Which permission to check for.
     *
     * @return mixed               Depending on the type of check,
     *                             either return a boolean to indicate
     *                             permission for that poll, or a
     *                             filtered out array of poll_ids.
     */
    function checkPerms()
    {
        return Auth::isAdmin('scry:admin');
    }

    function permissionsFilter($in, $filter, $permission = PERMS_READ)
    {
        // We purposely DO NOT use Auth::isAdmin('scry:admin') here,
        // since admins local to Scry still need to be granted
        // permission to the polls that they will administer.
        $admin = Scry::checkPerms();
        $out = array();
        require_once 'Horde/Group.php';
        $groups = &Group::singleton();
        $mygroups = $groups->getGroupMemberships(Auth::getAuth());

        switch ($filter) {
        case 'poll':
            if ($admin) {
                return $in;
            }
            foreach ($in as $key => $poll) {
                if (empty($poll['poll_group']) || isset($mygroups[$poll['poll_group']])) {
                    $out[$key] = $poll;
                }
            }
            break;
        }

        return $out;
    }

    function getMenu($returnType = 'object')
    {
        require_once 'Horde/Menu.php';

        $menu = &new Menu();

        /* Poll list. */
        $menu->add(Horde::applicationUrl('polls.php'), _("Polls"), 'scry.png', $GLOBALS['registry']->getImageDir());

        if (Auth::isAdmin('scry:admin')) {
            /* Create new poll. */
            $menu->add(Horde::applicationUrl('edit.php'), _("New Poll"), 'new.png', $GLOBALS['registry']->getImageDir());

            /* Export poll data. */
            if ($id = Util::getFormData('poll_id')) {
                $menu->add(Horde::applicationUrl(Util::addParameter('export.php', 'poll_id', $id)), _("Export"), 'data.png', $GLOBALS['registry']->getImageDir('horde'));
            }
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
